{-|
Module      : EPO.API.Client.Implementation
Description : EPO Patents database crawler (API client)
Copyright   : (c) CNRS, 2023-present
License     : AGPL + CECILL v3
Maintainer  : team@gargantext.org
Stability   : experimental
Portability : POSIX
-}

module EPO.API.Client.Implementation
  ( searchEPOAPI
  , searchEPOAPIC
  )
where

import Conduit
import EPO.API.Client
import EPO.API.Client.Types
import Network.URI qualified as URI
import Protolude
import Servant.Client (ClientError, runClientM)


batchSize :: Int
batchSize = 20


-- | This is a basic function for searching, don't use it for full
-- search, it fetches whole page at once.
searchEPOAPI :: URI.URI
             -> AuthKey
             -> Start
             -> End
             -> Search
             -> IO HyperdataDocuments
searchEPOAPI uri auth start end search' = do
  env <- defaultClientEnv uri
  eRes <- runClientM (search (Just auth) (Just start) (Just end) (Just search')) env
  case eRes of
    Left err-> throwIO err
    Right res -> pure res



searchEPOAPIC :: URI.URI
              -> AuthKey
              -> Maybe Start
              -> Maybe End
              -> Search
              -> IO (Either ClientError (Integer, ConduitT () HyperdataDocument IO ()))
searchEPOAPIC uri auth mStart mEnd search' = do
  let start = fromMaybe 1 mStart
  env <- defaultClientEnv uri
  eRes1 <- runClientM (search (Just auth) (Just 1) (Just 2) (Just search')) env
  case eRes1 of
    Left err -> pure $ Left err
    Right (Paginated { total }) -> do
      let end = fromMaybe total mEnd
      let fetch s = do
            let endS = minimum [s + batchSize - 1, end]
            eRes <- runClientM (search (Just auth) (Just s) (Just endS) (Just search')) env
            case eRes of
              Left err -> throwIO err
              Right (Paginated { items }) -> pure items
      pure $ Right (fromIntegral total
                   , yieldMany [start, start + batchSize .. end]
                     .| concatMapMC fetch)
