{-|
Module      : EPO.API.Client.Types
Description : EPO Patents API client
Copyright   : (c) CNRS, 2023-present
License     : AGPL + CECILL v3
Maintainer  : team@gargantext.org
Stability   : experimental
Portability : POSIX
-}


module EPO.API.Client.Types where

import Data.Aeson
import Data.LanguageCodes (ISO639_1)
import Data.Map.Strict qualified as Map
import Data.Text.Encoding qualified as TE
import Protolude
import Servant.API


type Start = Int
type End = Int
type Search = Text


data HyperdataDocument = HyperdataDocument
  { id                :: Maybe Text
  , titles            :: Map.Map ISO639_1 Text
  , authors           :: [Text]
  , abstracts         :: Map.Map ISO639_1 Text
  , publication_date  :: Maybe Text
  , publication_year  :: Maybe Int
  , publication_month :: Maybe Int
  , publication_day   :: Maybe Int
  } deriving (Show, Eq, Generic)
instance ToJSON HyperdataDocument
instance FromJSON HyperdataDocument


data Paginated a = Paginated
  { total :: Int
  , start :: Start
  , end   :: End
  , items :: [a]
  } deriving (Show, Eq, Generic)
instance ToJSON a => ToJSON (Paginated a)
instance FromJSON a => FromJSON (Paginated a)

type HyperdataDocuments = Paginated HyperdataDocument



newtype User = User { unUser :: Text }
  deriving (Eq, Show, Generic, IsString)
instance ToJSON User where
  toJSON (User u) = toJSON u

newtype Token = Token { unToken :: Text }
  deriving (Eq, Show, Generic, IsString)
instance ToJSON Token where
  toJSON (Token t) = toJSON t


data AuthKey = AuthKey { user  :: User
                       , token :: Token }
  deriving (Eq, Show, Generic)
joinKeys :: AuthKey -> Text
joinKeys (AuthKey { .. }) = unUser user <> ":" <> unToken token
instance ToHttpApiData AuthKey where
  toUrlPiece (AuthKey { }) = "not implemented"
  toHeader authKey = TE.encodeUtf8 $ "Basic " <> (joinKeys authKey)

